package ALTLinux::RepoList;

use strict;
use warnings;
use Carp;
#use Source::Shared::FindLocalMirror::ALTLinux;

require Exporter;
our @ISA = qw(Exporter);
our @EXPORT_OK = qw(load_listfile_lambda_fn listdirs2srclists listdirs2binlists is_listuri_in_repo_tree listuri2reporoot);

our $VERSION=0.006;
#our $verbose=0;
#$verbose=1 if $ENV{TERM}; # not to pollute cron mail


# public ------------------------------------------------------------

sub load_listfile_lambda_fn {
    my ($list, $lambdafunc_fn)=@_;
    my $fn;
    if ($list=~/\.xz$/) {
	open ($fn, '-|', 'xzcat', $list) or die "can't xzcat file $list\n";
    } elsif ($list=~/\.gz$/) {
	open ($fn, '-|', 'zcat', $list) or die "can't zcat file $list\n";
    } elsif ($list=~/\.bz2$/) {
	open ($fn, '-|', 'bzcat', $list) or die "can't bzcat file $list\n";
    } else {
	open ($fn, '<', $list) or die "can't open file $list\n";
    }
    my @ret=&$lambdafunc_fn($fn);
    close($fn);
    return @ret;
}

sub listdirs2srclists {
    return map {&__find_file_plain_or_comressed($_,'src.list')} @_;
}

sub listdirs2binlists {
    return map {&__find_file_plain_or_comressed($_,'bin.list')} @_;
}

sub is_listuri_in_repo_tree {
    my $altlist=shift;
    return $altlist =~ m!/files/list/(?:src|bin).list(\.(?:xz|gz|bz2))?$!;
}

# use only if is_listuri_in_repo_tree true!
sub listuri2reporoot {
    my $list_uri=shift;
    confess "ALTLinux::RepoList: Internal error: undefined list" if (! defined $list_uri);
    my $reporoot = $list_uri=~s!/files/list/(?:src|bin).list(\.(?:xz|gz|bz2))?$!!r;
    confess "ALTLinux::RepoList: Internal error: list not in repo: $list_uri" if (! defined $reporoot);
    return $reporoot;
}

# protected ------------------------------------------------------------

sub process_bin_lists {
    my ($lambda, @binlists)=@_;
    return map {&__process_bin_list($_,$lambda)} @binlists;
}

sub process_src_lists {
    my ($lambda, @srclists)=@_;
    return map {&__process_src_list($_,$lambda)} @srclists;
}

# private ------------------------------------------------------------


sub __process_bin_list {
    my ($list,$lambda)=@_;
    my $root=&listuri2reporoot($list);
    return &__load_listfile_lambda2($list,\&__read_altbinlist_fn_lambda,$lambda,$root);
}


sub __process_src_list {
    my ($list,$lambda)=@_;
    my $root=&listuri2reporoot($list);
    return &__load_listfile_lambda2($list,\&__read_altsrclist_fn_lambda,$lambda,$root);
}

sub __find_file_plain_or_comressed {
    my ($dir, $file)=@_;
    my $plain=$_.'/'.$file;
    return $plain if -e $plain;
    my $compressed=$plain.'.xz';
    return $compressed if -e $compressed;
    $compressed=$plain.'.gz';
    return $compressed if -e $compressed;
    $compressed=$plain.'.bz2';
    return $compressed if -e $compressed;
    return;
}

sub __load_listfile_lambda2 {
    my ($list, $readfunc,$linefunc, $reporoot)=@_;
    return &load_listfile_lambda_fn(
	$list, sub {
	    my $fn=shift;
	    &$readfunc($fn,$linefunc,$reporoot);
	}
	);
}

sub __read_altsrclist_fn_lambda {
    my ($fn,$lambda,$reporoot)=@_;
    while (my $line=<$fn>) {
	#warn 'got line: ', $line;
	chomp $line;
	my @list = split(/\s+/,$line);
	next if @list!=3;
	my ($name,$evr,$srpm)=@list;
	&$lambda($name,$evr,$srpm,$reporoot);
    }
}

sub __read_altbinlist_fn_lambda {
    my ($fn,$lambda,$reporoot)=@_;
    while (my $line=<$fn>) {
	chomp $line;
	my @list = split(/\s+/,$line);
	next if @list!=5;
	my ($name,$evr,$arch,$rpm,$srpm)=@list;
	&$lambda($name,$evr,$arch,$rpm,$srpm,$reporoot);
    }
}

# ------------------------------------------------------------

1;
